/*=========================================================================

Program:   ORFEO Toolbox
Language:  C++
Date:      $Date$
Version:   $Revision$


Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbViewerEducationController_h
#define __otbViewerEducationController_h

#include "itkObject.h"

#include "otbViewerEducationControllerInterface.h"
#include "otbViewerEducationView.h"
#include "otbViewerEducationModel.h"

#include "otbWidgetResizingActionHandler.h"
#include "otbChangeScaledExtractRegionActionHandler.h"
#include "otbChangeExtractRegionActionHandler.h"
#include "otbChangeScaleActionHandler.h"
#include "otbPixelDescriptionActionHandler.h"

namespace otb
{
class ViewerEducationController
  : public ViewerEducationControllerInterface
{
public:
  /** Standard class typedefs */
  typedef ViewerEducationController         Self;
  typedef itk::Object                       Superclass;
  typedef itk::SmartPointer<Self>           Pointer;
  typedef itk::SmartPointer<const Self>     ConstPointer;

  /** Standard type macros */
  itkTypeMacro(ViewerEducationController, Superclass);
  itkNewMacro(Self);

  typedef ViewerEducationModel ModelType;

  /** Widgets controller and action handlers */
  typedef ViewerEducationView::ImageViewType              ImageViewType;
  typedef ViewerEducationModel::VisualizationModelType    VisualizationModelType;
  typedef ViewerEducationModel::PixelDescriptionModelType PixelDescriptionModelType;

  typedef ImageWidgetController                    WidgetsControllerType;
  typedef std::map<std::string, 
                   WidgetsControllerType::Pointer> ControllerMapType;
  
  typedef WidgetResizingActionHandler
    <VisualizationModelType, ImageViewType>                      ResizingHandlerType;
  typedef ChangeScaledExtractRegionActionHandler
  <VisualizationModelType, ImageViewType>                        ChangeScaledRegionHandlerType;
  typedef ChangeExtractRegionActionHandler
  <VisualizationModelType, ImageViewType>                        ChangeRegionHandlerType;
  typedef ChangeScaleActionHandler
  <VisualizationModelType, ImageViewType>                        ChangeScaleHandlerType;
  typedef otb::PixelDescriptionActionHandler
  < PixelDescriptionModelType, ImageViewType>                   PixelDescriptionActionHandlerType;

  
  /** User action */
  virtual void Save(const std::string &);
  virtual void ExportQGIS(const std::string &, const std::string &);
  virtual void ExportKMZ(const std::string &);
  virtual void SetTrueColorComposition(const std::string & layerId);
  virtual void SetGrayColorComposition(const std::string & layerId, 
                                       const unsigned int &channel,
                                       const std::string & desc);
  virtual void KMeans(const std::string& layerId, unsigned int nbClasses);
  virtual void NDVI(const std::string & layerId);
  virtual void Brightness(const std::string & layerId);

  virtual void InitializeController();
  virtual void WriteSpreadsheetFile(const std::string &);
  
  /** Set the pointer to the view */
  virtual void SetView(ViewerEducationView * view);

  virtual void SetModel(ViewerEducationModel* model);

  /** Get the widgets controller */
  itkGetObjectMacro(WidgetsController, WidgetsControllerType);

  /** Get the controller map */
  virtual ControllerMapType  GetControllerMap()
  {
    return m_ControllerMap;
  }

  /** Create a new controller and add it to the map */
  virtual void AddControllerToMap(const std::string& layerId);

  /** Link the current added layer to the main view */
  void LinkToMainDisplay(const std::string& layerId);

  /** Get the input image mode*/
  virtual unsigned int GetInputImageMode(const std::string& infname);

protected:
  /** Constructor */
  ViewerEducationController();
  /** Destructor */
  ~ViewerEducationController();

  /** Link two display using their layerId */
  void LinkDispalys(const std::string& layerIdReference, 
                    const std::string& layerIdSecondary);

  void InitializeWidgetsController();
  
  void LinkSubViewers();

private:
  ViewerEducationController(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented

  /** Pointer to the view */
  ViewerEducationView::Pointer               m_View;

  /** Widgets controller */
  WidgetsControllerType::Pointer             m_WidgetsController;

  /** Action handlers */
  ResizingHandlerType::Pointer               m_ResizingHandler;
  ChangeScaleHandlerType::Pointer            m_ChangeScaleHandler;
  ChangeRegionHandlerType::Pointer           m_ChangeRegionHandler;
  ChangeScaledRegionHandlerType::Pointer     m_ChangeScaledRegionHandler;
  PixelDescriptionActionHandlerType::Pointer m_PixelDescriptionHandler;

  ControllerMapType                          m_ControllerMap;

  ViewerEducationModel::Pointer             m_Model;

  std::string                               m_LastLayerAdded;

};
} //end namespace otb

#endif
