/*=========================================================================

Program:   ORFEO Toolbox
Language:  C++
Date:      $Date$
Version:   $Revision$


Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbViewerEducationControllerInterface_h
#define __otbViewerEducationControllerInterface_h

#include "itkObject.h"
#include "otbViewerEducationModel.h"
#include "otbImageWidgetController.h"
#include "otbPixelSpectrumController.h"

namespace otb
{
class ViewerEducationControllerInterface
      : public itk::Object
{
public:
  /** Standard class typedefs */
  typedef ViewerEducationControllerInterface       Self;
  typedef itk::Object                              Superclass;
  typedef itk::SmartPointer<Self>                  Pointer;
  typedef itk::SmartPointer<const Self>            ConstPointer;

  typedef ImageWidgetController                    WidgetsControllerType;
  typedef std::map<std::string, 
                   WidgetsControllerType::Pointer> ControllerMapType;

  typedef otb::PixelSpectrumController         PixelSpectrumControllerType;
  typedef PixelSpectrumControllerType::Pointer PixelSpectrumControllerPointerType;
  
  /** Standard type macros */
  itkTypeMacro(ViewerEducationControllerInterface, Superclass);

  /** Users actions */
  virtual void Save(const std::string &) = 0;
  virtual void ExportQGIS(const std::string &, const std::string & ) = 0;
  virtual void ExportKMZ(const std::string &) = 0;
  virtual void SetTrueColorComposition(const std::string& layerId)  = 0;
  virtual void SetGrayColorComposition(const std::string& layerId,
                                       const unsigned int &channel,
                                       const std::string & desc)  = 0;
  virtual void KMeans(const std::string& layerId, unsigned int nbClasses)  = 0;
  virtual void NDVI(const std::string& layerId) = 0;
  virtual void Brightness(const std::string& layerId) = 0;
  
  virtual ControllerMapType  GetControllerMap() = 0;
  virtual void AddControllerToMap(const std::string& layerId) = 0;

  virtual void SetModel(ViewerEducationModel* model) = 0;

  virtual void InitializeController() = 0;
  virtual void WriteSpreadsheetFile(const std::string &) = 0;

  virtual unsigned int GetInputImageMode(const std::string& infname) = 0;
  
  itkSetObjectMacro(PixelSpectrumController, PixelSpectrumControllerType);
  itkGetObjectMacro(PixelSpectrumController, PixelSpectrumControllerType);

protected:
  /** Constructor */
  ViewerEducationControllerInterface()
  {}

  /** Destructor */
  ~ViewerEducationControllerInterface() {}

  /** The instance of the model (for commodity) */
  ViewerEducationModel::Pointer m_Model;
  PixelSpectrumControllerPointerType m_PixelSpectrumController;

private:
  ViewerEducationControllerInterface(const Self&); //purposely not implemented
  void operator=(const Self&); //purposely not implemented
  
};
} // end namespace otb

#endif
