/*=========================================================================

Program:   ORFEO Toolbox
Language:  C++
Date:      $Date$
Version:   $Revision$


Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
See OTBCopyright.txt for details.


This software is distributed WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the above copyright notices for more information.

=========================================================================*/

#ifndef __TypeDefinitions_h
#define __TypeDefinitions_h


#include "otbImage.h"
#include "otbVectorImage.h"
#include "otbImageFileReader.h"
#include "otbImageFileWriter.h"
#include "otbMultiChannelRAndNIRIndexImageFilter.h"
#include "otbVegetationIndicesFunctor.h"
#include "otbImageToVectorImageCastFilter.h"
#include "otbObjectList.h"

// Color mapping
#include "otbChangeLabelImageFilter.h"
#include "otbRGBFromImageValueFilter.h"

// KMeans
#include "itkListSample.h"
#include "otbKMeansFunctor.h"
#include "itkWeightedCentroidKdTreeGenerator.h"
#include "itkKdTreeBasedKmeansEstimator.h"
#include "itkMersenneTwisterRandomVariateGenerator.h"
#include "itkImageRegionConstIterator.h"
#include "itkUnaryFunctorImageFilter.h"

#include "otbStreamingShrinkImageFilter.h"

//Brightness
#include "otbSoilIndicesFunctor.h"
#include "otbMultiChannelGAndRIndexImageFilter.h"
#include "otbMultiChannelRAndGAndNIRIndexImageFilter.h"

//KMZ export
#include "otbMultiImageKmzProductWriter.h"

#include "otbStreamingMinMaxVectorImageFilter.h"


/** Algorithms typedef */

typedef unsigned int                      LabelType;
typedef otb::VectorImage<double,2>        VectorImageType;
typedef VectorImageType::PixelType        VectorPixelType;
typedef otb::Image<double,2>              ImageType;
typedef otb::Image<LabelType, 2>          LabelImageType;

typedef otb::ImageFileReader<VectorImageType> ReaderType;
typedef otb::ImageFileWriter<LabelImageType>           LabelImageWriterType;
typedef otb::ImageFileWriter<ImageType>           ImageWriterType;
typedef otb::Functor::NDVI<double,double,double> NDVIFunctorType;
typedef otb::MultiChannelRAndNIRIndexImageFilter<VectorImageType, ImageType, NDVIFunctorType> NDVIFilterType;
typedef otb::ImageToVectorImageCastFilter<LabelImageType, VectorImageType> LabelImageToVectorImageFilterType;
typedef otb::ImageToVectorImageCastFilter<ImageType, VectorImageType> ImageToVectorImageFilterType;

typedef itk::ImageRegionConstIterator<VectorImageType> IteratorType;
typedef itk::ImageRegionConstIterator<LabelImageType> LabeledIteratorType;
  
/** Color mapping typedef */
typedef otb::RGBFromImageValueFilter<VectorImageType, LabelImageType> RGBFromImageValueFilterType;
typedef otb::StreamingShrinkImageFilter
                    <VectorImageType, VectorImageType>                ImageSamplingFilterType;
typedef otb::ChangeLabelImageFilter<LabelImageType, 
                                 VectorImageType> ChangeLabelFilterType;

// map used to store association label/color
typedef std::map<unsigned int, VectorPixelType >            ColorMapRampType;

/** KMeans typedef*/
typedef itk::Statistics::ListSample<VectorImageType::PixelType>   ListSampleType;
typedef itk::Statistics::WeightedCentroidKdTreeGenerator<ListSampleType> TreeGeneratorType;
typedef TreeGeneratorType::KdTreeType TreeType;
typedef itk::Statistics::KdTreeBasedKmeansEstimator<TreeType> EstimatorType;

typedef otb::Functor::KMeansFunctor<VectorImageType::PixelType,   
                                 LabelType>                KMeansFunctorType;
typedef itk::UnaryFunctorImageFilter<VectorImageType,
                                       LabelImageType, 
                                       KMeansFunctorType>  KMeansFilterType;
                                       
/** Brightness */
typedef otb::Functor::IB2<VectorImageType::InternalPixelType, VectorImageType::InternalPixelType, 
                           VectorImageType::InternalPixelType, ImageType::PixelType>               IB2FunctorType;
typedef otb::Functor::IB<VectorImageType::InternalPixelType, VectorImageType::InternalPixelType, 
                        ImageType::PixelType>                                                     IBFunctorType;
typedef otb::MultiChannelRAndGAndNIRIndexImageFilter<VectorImageType,
                                                  ImageType,
                                                  IB2FunctorType>  IB2FilterType;
typedef otb::MultiChannelGAndRIndexImageFilter<VectorImageType,
                                                  ImageType,
                                                  IBFunctorType>  IBFilterType;
                                                  
/** KMZ export */
typedef otb::MultiImageKmzProductWriter<VectorImageType> KmzProductWriterType;
typedef otb::ObjectList<ReaderType> ReaderListType;

typedef otb::StreamingMinMaxVectorImageFilter<VectorImageType> StatFilterType;


#endif
