/*=========================================================================

 Program:   ORFEO Toolbox
 Language:  C++
 Date:      $Date$
 Version:   $Revision$


 Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
 See OTBCopyright.txt for details.


 This software is distributed WITHOUT ANY WARRANTY; without even
 the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the above copyright notices for more information.

 =========================================================================*/
#ifndef __otbRGBFromImageValueFunctor_h
#define __otbRGBFromImageValueFunctor_h

namespace otb
{

namespace Functor
{

// Functor to compute mean pixel value for each label
template <class TLabel, class TValue> 
class RGBFromImageValueFunctor
{
public:
  typedef std::map<TLabel, TValue> MeanValueMapType;

  void SetMaxVal(const TValue maxVal)
  {
    m_MaxVal.SetSize(maxVal.Size());
    for (unsigned int index = 0; index < maxVal.Size(); index++)
      {
      m_MaxVal.SetElement(index, maxVal.GetElement(index));
      }
  }

  void SetMinVal(const TValue minVal)
  {
    m_MinVal.SetSize(minVal.Size());
    for (unsigned int index = 0; index < minVal.Size(); index++)
      {
      m_MinVal.SetElement(index, minVal.GetElement(index));
      }
  }

  TValue GetMaxVal()
  {
    return m_MaxVal;
  }

  TValue GetMinVal()
  {
    return m_MinVal;
  }

  void AddNewLabel(TLabel label, TValue value)
  {
    TValue newValue;
    newValue.SetSize(value.Size());
    for (unsigned int index = 0; index < value.Size(); index++)
      {
      if (value[index] < m_MinVal[index])
        newValue[index] = m_MinVal[index];
      else
        if (value[index] > m_MaxVal[index])
          newValue[index] = m_MaxVal[index];
        else newValue[index] = value[index];
      }
    m_LabelToImageIntensityMap[label] = newValue;
    m_WeigthingMap[label] = 1;
  }

  void AddValue(const TLabel& label, const TValue& value)
  {
    TValue currentValue = m_LabelToImageIntensityMap[label];
    m_WeigthingMap[label] = m_WeigthingMap[label] + 1;
    for (unsigned int index = 0; index < value.Size(); index++)
      {
      if (value[index] < m_MinVal[index])
        currentValue[index] += m_MinVal[index];
      else
        if (value[index] > m_MaxVal[index])
          currentValue[index] += m_MaxVal[index];
        else currentValue[index] += value[index];

        }

    m_LabelToImageIntensityMap[label] = currentValue;
  }

  /** operator */
  TLabel operator ()(const TLabel& label, const TValue& value)
  {
    if (m_LabelToImageIntensityMap.count(label)<=0)
      {
      AddNewLabel(label, value);
        }
      else
        {
        AddValue(label, value);
        }

    return label;
  }

  MeanValueMapType GetMeanIntensity()
  {
    MeanValueMapType MeanMap;

    for(unsigned int i=0; i<m_WeigthingMap.size(); i++)
      {
      TValue value = m_LabelToImageIntensityMap[i];
      for (unsigned int index = 0; index < value.Size(); index++)
           {
            value[index]=value[index]/static_cast<float>(m_WeigthingMap[i]);
           }
      MeanMap[i]=value;
      }

    typename MeanValueMapType::iterator it;
    for(it = MeanMap.begin(); it != MeanMap.end(); it++)
    {
       std::cout<<"in functor : label = "<<it->first<<" , value = "<<it->second<< " ,nbPixels = "<< m_WeigthingMap[it->first] <<std::endl;
    }
    return MeanMap;
  }

  std::map<TLabel, unsigned int> GetLabelArea()
  {
    return m_WeigthingMap;
  }

  /** Constructor */
  RGBFromImageValueFunctor()
  {
  }

  bool operator !=(const RGBFromImageValueFunctor& other) const
  {

    return ((&m_LabelToImageIntensityMap) != &(other.m_LabelToImageIntensityMap));
  }
private:

  std::map<TLabel, unsigned int> m_WeigthingMap; //counter
  MeanValueMapType m_LabelToImageIntensityMap;
  TValue m_MinVal;
  TValue m_MaxVal;

};


} // end namespace functor

}// end namespace otb

#endif
