/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#include "otbSimplePackedWidgetManager.h"

namespace otb
{

SimplePackedWidgetManager
::SimplePackedWidgetManager()
{
  m_SimplePackedWidget = new ImageWidgetSimplePackedManagerType();
}

SimplePackedWidgetManager
::~SimplePackedWidgetManager()
{
  delete m_SimplePackedWidget;
}

void
SimplePackedWidgetManager
::RegisterFullWidget(WidgetType* fullWidget)
{
  m_InternalFullWidget = fullWidget;
  m_SimplePackedWidget->m_FullGroup->add(fullWidget);
  m_SimplePackedWidget->m_FullGroup->resizable(fullWidget);
  fullWidget->resize(m_SimplePackedWidget->m_FullGroup->x(), m_SimplePackedWidget->m_FullGroup->y(),
                     m_SimplePackedWidget->m_FullGroup->w(), m_SimplePackedWidget->m_FullGroup->h());
}

void
SimplePackedWidgetManager
::RegisterScrollWidget(WidgetType* scrollWidget)
{
  m_InternalScrollWidget = scrollWidget;
  m_SimplePackedWidget->m_QuicklookGroup->add(scrollWidget);
  m_SimplePackedWidget->m_QuicklookGroup->resizable(scrollWidget);
  scrollWidget->resize(m_SimplePackedWidget->m_QuicklookGroup->x(), m_SimplePackedWidget->m_QuicklookGroup->y(),
                       m_SimplePackedWidget->m_QuicklookGroup->w(), m_SimplePackedWidget->m_QuicklookGroup->h());
}

void
SimplePackedWidgetManager
::RegisterZoomWidget(WidgetType* zoomWidget)
{
  m_InternalZoomWidget = zoomWidget;
  m_SimplePackedWidget->m_ZoomGroup->add(zoomWidget);
  m_SimplePackedWidget->m_ZoomGroup->resizable(zoomWidget);
  zoomWidget->resize(m_SimplePackedWidget->m_ZoomGroup->x(), m_SimplePackedWidget->m_ZoomGroup->y(),
                     m_SimplePackedWidget->m_ZoomGroup->w(), m_SimplePackedWidget->m_ZoomGroup->h());
}
void
SimplePackedWidgetManager
::RegisterPixelDescriptionWidget(Fl_Output * pixelWidget)
{
  m_IntenalPixelWidget = pixelWidget;
  m_SimplePackedWidget->m_PixelDescriptionGroup->add(pixelWidget);
  m_SimplePackedWidget->m_PixelDescriptionGroup->resizable(pixelWidget);
  pixelWidget->resize(m_SimplePackedWidget->m_PixelDescriptionGroup->x(), m_SimplePackedWidget->m_PixelDescriptionGroup->y(),
                      m_SimplePackedWidget->m_PixelDescriptionGroup->w(), m_SimplePackedWidget->m_PixelDescriptionGroup->h());

}

void
SimplePackedWidgetManager
::RegisterHistogramWidget(WidgetType* curveWidget)
{
  m_InternalCurveWidget = curveWidget;
  m_SimplePackedWidget->m_HistogramsGroup->add(curveWidget);
  m_SimplePackedWidget->m_HistogramsGroup->resizable(curveWidget);
  curveWidget->resize(m_SimplePackedWidget->m_HistogramsGroup->x(), m_SimplePackedWidget->m_HistogramsGroup->y(),
                      m_SimplePackedWidget->m_HistogramsGroup->w(), m_SimplePackedWidget->m_HistogramsGroup->h());
}

void
SimplePackedWidgetManager
::Show()
{
  m_SimplePackedWidget->m_Window->show();
  m_InternalCurveWidget->show();
  m_IntenalPixelWidget->show();
  m_InternalFullWidget->show();
  m_InternalScrollWidget->show();
  m_InternalZoomWidget->show();
}

void
SimplePackedWidgetManager
::Hide()
{
  m_SimplePackedWidget->m_Window->hide();
}

void
SimplePackedWidgetManager
::Refresh()
{
  m_InternalScrollWidget->redraw();
  m_InternalFullWidget->redraw();
  m_InternalZoomWidget->redraw();
  m_IntenalPixelWidget->redraw();
  m_InternalCurveWidget->redraw();
}

void
SimplePackedWidgetManager
::UnRegisterAll()
{
  m_SimplePackedWidget->m_FullGroup->remove(m_InternalFullWidget);
  m_SimplePackedWidget->m_ZoomGroup->remove(m_InternalZoomWidget);
  m_SimplePackedWidget->m_PixelDescriptionGroup->remove(m_IntenalPixelWidget);
  m_SimplePackedWidget->m_QuicklookGroup->remove(m_InternalScrollWidget);
  m_SimplePackedWidget->m_HistogramsGroup->remove(m_InternalCurveWidget);
}

void
SimplePackedWidgetManager
::SetLabel(const char * label)
{
  m_SimplePackedWidget->m_Window->copy_label(label);
}

void
SimplePackedWidgetManager
::SetLabelScrollWidget(const char * label)
{
  m_SimplePackedWidget->m_QuicklookGroup->label(label);
}

void
SimplePackedWidgetManager
::SetLabelFullWidget(const char * label)
{
  m_SimplePackedWidget->m_FullGroup->label(label);
}

void
SimplePackedWidgetManager
::SetLabelZoomWidget(const char * label)
{
  m_SimplePackedWidget->m_ZoomGroup->label(label);
}

void
SimplePackedWidgetManager
::SetLabelHistogramWidget(const char * label)
{
  m_SimplePackedWidget->m_HistogramsGroup->label(label);
}

void
SimplePackedWidgetManager
::SetLabelPixelDescriptionWidget(const char * label)
{
  m_SimplePackedWidget->m_PixelDescriptionGroup->label(label);
}

int
SimplePackedWidgetManager
::Shown()
{
  return m_SimplePackedWidget->m_Window->shown();
}

void
SimplePackedWidgetManager
::SetResizable(bool resize)
{
  if (resize)
    {
    m_SimplePackedWidget->m_Window->resizable(m_SimplePackedWidget->m_Window);
    }
  else
    {
    // If the resizable is NULL then all widgets remain a fixed size
    m_SimplePackedWidget->m_Window->resizable(NULL);
    }
}

void
SimplePackedWidgetManager
::AddCallback(Fl_Callback callback, void * userData)
{
  return m_SimplePackedWidget->m_Window->callback(callback, userData);
}

void
SimplePackedWidgetManager
::ResizeFull(double w, double h)
{
  m_SimplePackedWidget->m_FullGroup->resizable(NULL);
  m_InternalFullWidget->resizable(NULL);
  m_InternalFullWidget->size_range(w,h,w,h);

  m_InternalFullWidget->size(w, h);
  m_SimplePackedWidget->m_FullGroup->size(w, h);
}

void
SimplePackedWidgetManager
::ResizeZoom(double w, double h)
{
  m_InternalZoomWidget->resizable(NULL);
  m_InternalZoomWidget->size_range(w,h,w,h);
  m_SimplePackedWidget->m_ZoomGroup->resizable(NULL);

  m_InternalZoomWidget->size(w, h);
  m_SimplePackedWidget->m_ZoomGroup->size(w, h);
}


void
SimplePackedWidgetManager
::ResizeScroll(double w, double h)
{
  m_InternalScrollWidget->resizable(NULL);
  m_InternalScrollWidget->size_range(w,h,w,h);
  m_SimplePackedWidget->m_QuicklookGroup->resizable(NULL);

  m_InternalScrollWidget->size(w, h);
  m_SimplePackedWidget->m_QuicklookGroup->size(w, h);
}

void
SimplePackedWidgetManager
::ResizeHistogramCurve(double w, double h)
{
  m_InternalCurveWidget->resizable(NULL);
  m_SimplePackedWidget->m_HistogramsGroup->resizable(NULL);
  m_InternalCurveWidget->size_range(w,h,w,h);

  m_InternalCurveWidget->size(w, h);
  m_SimplePackedWidget->m_HistogramsGroup->size(w, h);
}

void
SimplePackedWidgetManager
::ResizePixelDescription(double w, double h)
{
  m_SimplePackedWidget->m_PixelDescriptionGroup->resizable(NULL);
  m_IntenalPixelWidget->size(w,h);
  m_SimplePackedWidget->m_PixelDescriptionGroup->size(w,h);
}

void
SimplePackedWidgetManager
::AdjustWidgets()
{
  // Remember the resizable status to put it back at the end of the
  // resize
  bool wasResizable = false;
  if (m_SimplePackedWidget->m_Window->resizable() != NULL)
    {
    wasResizable = true;
    }
  else
    {
    m_SimplePackedWidget->m_Window->size_range(1,1,
                                         10000,10000);
    m_SimplePackedWidget->m_Window->resizable(m_SimplePackedWidget->m_Window);
    }

  // Compute the size of the wMainWindow following the size of the
  // FullWindow, resize the wMainWindow, and replace the other widgets
  double marginX = m_InternalScrollWidget->x();
  double marginY = m_InternalScrollWidget->y();

  // adjust theFull
  short fullX = (short)(marginX + m_InternalScrollWidget->w() + marginX);
  short fullY = marginY;

  // Compute the new size of the mainWindow
  double newWidth = fullX + m_InternalFullWidget->w() + marginX;
  double newHeight = marginY + m_InternalFullWidget->h() + 8;
  m_SimplePackedWidget->m_Window->size(newWidth, newHeight);
  
  //Constraint the size to make it not resizable
  if (!wasResizable)
    {
    m_SimplePackedWidget->m_Window->size_range(newWidth,newHeight,
                                               newWidth,newHeight);
    m_SimplePackedWidget->m_Window->resizable(NULL);
    }

  // Recompute the margins, since the wMainWindow was resized, those
  // margins could change
  marginX = m_InternalScrollWidget->x();
  marginY = m_InternalScrollWidget->y();

  // Make sure that the scroll is well resized
  this->ResizeScroll(m_InternalScrollWidget->w(), m_InternalScrollWidget->h());
  m_SimplePackedWidget->m_QuicklookGroup->position(marginX, marginY);
  m_InternalScrollWidget->position(marginX, marginY);
  
  // adjust the Zoom
  short zoomX = marginX;
  short zoomY = marginY + m_InternalScrollWidget->h() + marginY;

  this->ResizeZoom(m_InternalZoomWidget->w(), m_InternalZoomWidget->h());
  m_InternalZoomWidget->position(zoomX, zoomY);
  m_SimplePackedWidget->m_ZoomGroup->position(zoomX, zoomY);

  // PrintWidgetSizeAndPosition(m_InternalZoomWidget,"m_InternalZoomWidget");
  // PrintWidgetSizeAndPosition(m_SimplePackedWidget->m_ZoomGroup,"m_InternalZoomWidget");
  
  // adjust the Histogram
  short histoX = marginX;
  short histoY = m_InternalZoomWidget->y() + m_InternalZoomWidget->h() + marginY; 

  this->ResizeHistogramCurve(m_InternalCurveWidget->w(),m_InternalCurveWidget->h());
  m_InternalCurveWidget->position(histoX, histoY);
  m_SimplePackedWidget->m_HistogramsGroup->position(histoX, histoY);

  // adjust the Pixel Description
  short pixelX = marginX;
  short pixelY = m_InternalCurveWidget->y() + m_InternalCurveWidget->h() + marginY; 

  this->ResizePixelDescription(m_IntenalPixelWidget->w(), m_IntenalPixelWidget->h());
  m_IntenalPixelWidget->position(pixelX, pixelY);
  m_SimplePackedWidget->m_PixelDescriptionGroup->position(pixelX, pixelY);

  // then, recompute the full coordinates
  fullX = (short)(marginX + m_InternalScrollWidget->w() + marginX);
  fullY = marginY;

  // Adjust the Full Widget
  this->ResizeFull(m_InternalFullWidget->w(), m_InternalFullWidget->h());
  m_SimplePackedWidget->m_FullGroup->position(fullX, fullY);
  m_InternalFullWidget->position(fullX, fullY);

  // // Verbosity
  // std::cout <<"\t After mainWindow Resize to "<< newWidth << ","<< newHeight << std::endl;
  // PrintWidgetSizeAndPosition(m_SimplePackedWidget->m_Window,"m_SimplePackedWidget->m_Window");
  // PrintWidgetSizeAndPosition(m_InternalFullWidget,"m_InternalFullWidget");
  // PrintWidgetSizeAndPosition( m_SimplePackedWidget->m_FullGroup," m_SimplePackedWidget->m_FullGroup");

  // PrintWidgetSizeAndPosition(m_InternalScrollWidget,"m_InternalScrollWidget");
  // PrintWidgetSizeAndPosition(m_SimplePackedWidget->m_QuicklookGroup,"m_SimplePackedWidget->m_QuicklookGroup");

  // PrintWidgetSizeAndPosition(m_InternalZoomWidget,"m_InternalZoomWidget");
  // PrintWidgetSizeAndPosition(m_SimplePackedWidget->m_ZoomGroup,"m_SimplePackedWidget->m_ZoomGroup");

  // //PrintWidgetSizeAndPosition(m_IntenalPixelWidget,"m_IntenalPixelWidget");
  // PrintWidgetSizeAndPosition(m_SimplePackedWidget->m_PixelDescriptionGroup,"m_SimplePackedWidget->m_PixelDescriptionGroup");

}


void
SimplePackedWidgetManager
::PrintWidgetSizeAndPosition(WidgetType* widget, const std::string widgetId)
{

  std::cout <<"Widget : "<< widgetId << " Pos : "<< widget->x() <<","<<widget->y() 
            << " And Size : "<< widget->w()<< ","<< widget->h()<< std::endl;
}

void
SimplePackedWidgetManager
::PrintWidgetSizeAndPosition(Fl_Double_Window* widget, const std::string widgetId)
{

  std::cout <<"Widget : "<< widgetId << " Pos : "<< widget->x() <<","<<widget->y() 
            << " And Size : "<< widget->w()<< ","<< widget->h()<< std::endl;
}

void
SimplePackedWidgetManager
::PrintWidgetSizeAndPosition(Fl_Group* widget, const std::string widgetId)
{

  std::cout <<"Widget : "<< widgetId << " Pos : "<< widget->x() <<","<<widget->y() 
            << " And Size : "<< widget->w()<< ","<< widget->h()<< std::endl;
}


}
