/*=========================================================================

  Program:   ORFEO Toolbox
  Language:  C++
  Date:      $Date$
  Version:   $Revision$


  Copyright (c) Centre National d'Etudes Spatiales. All rights reserved.
  See OTBCopyright.txt for details.


     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#ifndef __otbSimplePackedWidgetManager_h
#define __otbSimplePackedWidgetManager_h

#include "otbWidgetManager.h"
#include "otbImageWidgetSimplePackedManager.h"

#include <FL/Fl_Double_Window.H>

namespace otb
{
/** \class SimplePackedWidgetManager
 *   \brief todo
 *
 *  \ingroup Visualization
 */
class SimplePackedWidgetManager
  : public WidgetManager
{
public:
  /** Standard class typedefs */
  typedef SimplePackedWidgetManager           Self;
  typedef WidgetManager                 Superclass;
  typedef itk::SmartPointer<Self>       Pointer;
  typedef itk::SmartPointer<const Self> ConstPointer;

  /** Method for creation through the object factory */
  itkNewMacro(Self);

  /** Runtime information */
  itkTypeMacro(SimplePackedWidgetManager, WidgetManager);

  /** typedef of the fltk file defining all the widget needed*/
  typedef ImageWidgetSimplePackedManager ImageWidgetSimplePackedManagerType;

  /** Register Wigets*/
  virtual void RegisterFullWidget(WidgetType* mfullWidget);
  virtual void RegisterScrollWidget(WidgetType* scrollWidget);
  virtual void RegisterZoomWidget(WidgetType* zoomWidget);
  virtual void RegisterHistogramWidget(WidgetType* curveWidget);
  virtual void RegisterPixelDescriptionWidget(Fl_Output * pixelWidget);
  virtual void Show();
  virtual void Hide();
  virtual void Refresh();
  virtual void SetLabel(const char * label);
  virtual void UnRegisterAll();
  virtual void SetLabelScrollWidget(const char * label);
  virtual void SetLabelFullWidget(const char * label);
  virtual void SetLabelZoomWidget(const char * label);
  virtual void SetLabelHistogramWidget(const char * label);
  virtual void SetLabelPixelDescriptionWidget(const char * label);
  virtual int  Shown();
  virtual void SetResizable(bool resize);

  virtual void ResizeFull(double w, double h);
  virtual void ResizeZoom(double w, double h);
  virtual void ResizeScroll(double w, double h);
  virtual void ResizeHistogramCurve(double w, double h);
  virtual void ResizePixelDescription(double w, double h);
  virtual void AdjustWidgets();

  /** Add callback to the main window. Pay attention about the fact
    * that only one callback is allowed by widget, so adding a callback
    * using this method will replace any other callback associated to the
    * main window, even the default one which is close the window using
    * the cross
    */
  void AddCallback(Fl_Callback callback, void * userData);

protected:
  /** Constructor */
  SimplePackedWidgetManager();

  /** Destructor */
  virtual ~SimplePackedWidgetManager();
  /** Printself method */
  void PrintSelf(std::ostream& os, itk::Indent indent) const
  {
    Superclass::PrintSelf(os, indent);
  }
  
  void PrintWidgetSizeAndPosition(WidgetType* widget, const std::string widgetId);
  void PrintWidgetSizeAndPosition(Fl_Double_Window* widget, const std::string widgetId);
  void PrintWidgetSizeAndPosition(Fl_Group* widget, const std::string widgetId);

private:
  SimplePackedWidgetManager(const Self&);     // purposely not implemented
  void operator =(const Self&); // purposely not implemented

  ImageWidgetSimplePackedManagerType* m_SimplePackedWidget;

};
}
#endif
